package de.tub.mapc.Coordination.goals;

import java.io.IOException;

import de.dailab.keks.io.IDeserialiser;
import de.dailab.keks.io.ISerialiser;
import de.tub.mapc.IMAPCConstants;
import de.tub.mapc.components.contestagents.MessageService;
import de.tub.mapc.components.contestagents.handle.IPathFindingHandle;
import de.tub.mapc.knowledge.Gridworld;
import de.tub.mapc.knowledge.Perception;
import de.tub.mapc.knowledge.Position;
import de.tub.mapc.knowledge.Switch;
import de.tub.mapc.pathfinding.PathResult;
import de.tub.mapc.worldmodel.GridField;

public class GoalFindSwitchForFence extends AbstractGoal {

	private Position lastKnownFenceField;
	private boolean isReached;
	
	public GoalFindSwitchForFence(){
		
	}
	
	public GoalFindSwitchForFence(Position fencepos){
		lastKnownFenceField = fencepos;
	}
	
	public Position getFenceField(){
		return lastKnownFenceField;
	}
	
	@Override
	public boolean equals(Object o) {
		if(o == null){
			return false;
		}
		if(o instanceof GoalFindSwitchForFence){
			GoalFindSwitchForFence other = (GoalFindSwitchForFence) o;
			//TODO ist das wirklich richtig? hier veraendert sich das feld ja staendig
			if (other.lastKnownFenceField.equals(lastKnownFenceField)){
				return true;
			}
		}
		return false;
	}

	@Override
	public String getAction(Perception perception, Gridworld world,
			IPathFindingHandle pathfinder,
			MessageService coordinatorMessageService) {
		
		
		boolean foundSwitch = (null != world.findSwitchForFence(lastKnownFenceField));
		if (foundSwitch){
			isReached = true;
			return IMAPCConstants.ACTION_SKIP; //TODO welche aktion wenn switch gefunden wurde?
		}
		
		// unbekanntes Ende des Zaunes suchen und direkt dahin laufen
		updateLastFenceField(world);
		// pathfinder befragen
		PathResult pr = pathfinder.getPath(lastKnownFenceField, perception.getAgentsPosition(), world, false);
		if(pr.isReachedGoal()){
			// this should not happen, the destination is updated in every step he explores something of the fence
			isReached = true;
			return IMAPCConstants.ACTION_SKIP;
			
		}else if(pr.getErrortype() != null){
						
			if (pr.getErrortype() == IMAPCConstants.PATH_ERROR){
				// there was an real error in the pathfinder
				// this should not happen
				// but when, the agent should move on
				// -> random move
				System.err.println("PATH: THERE WAS AN ERROR");
				return this.getRandomAction();
			}else if (pr.getErrortype() == IMAPCConstants.PATH_NOTFOUND){
				// the pathfinder was not able to find a path
				// should not happen, because this goal is only assigned if there is an path 
				return this.getRandomAction();
			
			}else if (pr.getErrortype() == IMAPCConstants.PATH_FENCEAHEAD){
				// the pathfinder indicates there is a fence in the next move,
				// maybe we already voted but the switcher hasn't reached the switching position yet or we didn't vote
				Switch s = world.findSwitchForFence(new Position(pr.getFencex(),pr.getFencey()));
				if(s == null){
					// there is no switch found yet
					coordinatorMessageService.sendSwitchVote(new Position(pr.getFencex(), pr.getFencey()),true, pr.getStepsToNextFence());
				}else{
					// vote for this switch to be opened
					coordinatorMessageService.sendSwitchVote(s, false, pr.getStepsToNextFence());
				}
				// If the fence is in the next move, the action is skip
				return pr.getThisStep();
			}else if (pr.getErrortype() == IMAPCConstants.PATH_BLOCKED){
				// The Pathfinder indicates the next move is blocked
				// da is ne kuh oder nen agent ins bild gerannt, 
				return this.getRandomAction();
			}
		}else{
			// no error, vote if there are fences ahead
			if(pr.getStepsToNextFence() != -1){
				Position s = world.findSwitchForFence(new Position(pr.getFencex(), pr.getFencey()));
				if(s == null){
					// there is no switch found yet
					coordinatorMessageService.sendSwitchVote(new Position(pr.getFencex(),pr.getFencey()),true, pr.getStepsToNextFence());
				}else{
					// vote for this switch to be opened
					coordinatorMessageService.sendSwitchVote(s, false, pr.getStepsToNextFence());
				}
			}
			// TODO es koennte sein, dass keiner erreicht ohne durch Zaun zu gehen
			// koennen wir uns aber spaeter Gedanken drueben machen
			return pr.getThisStep();
		}
		
		return this.getRandomAction();
	}

	private void updateLastFenceField(Gridworld world) {
		// unbekanntes ende des Zaunes finden
		GridField[][] grid  = world.getGrid();
		int x = lastKnownFenceField.getX();
		int y = lastKnownFenceField.getY();
		// search west
		while(grid[x][y].getType() == IMAPCConstants.TYPE_FENCE){
			x--;
		}
		if(grid[x][y].getType() == IMAPCConstants.TYPE_UNKNOWN){
			lastKnownFenceField.setX(x);
			return;
		}
		x = lastKnownFenceField.getX();
		y = lastKnownFenceField.getY();
		// search east
		while(grid[x][y].getType() == IMAPCConstants.TYPE_FENCE){
			x++;
		}
		if(grid[x][y].getType() == IMAPCConstants.TYPE_UNKNOWN){
			lastKnownFenceField.setX(x);
			return;
		}
		x = lastKnownFenceField.getX();
		y = lastKnownFenceField.getY();
		// search north
		while(grid[x][y].getType() == IMAPCConstants.TYPE_FENCE){
			y--;
		}
		if(grid[x][y].getType() == IMAPCConstants.TYPE_UNKNOWN){
			lastKnownFenceField.setY(y);
			return;
		}
		x = lastKnownFenceField.getX();
		y = lastKnownFenceField.getY();
		// search south
		while(grid[x][y].getType() == IMAPCConstants.TYPE_FENCE){
			y++;
		}
		if(grid[x][y].getType() == IMAPCConstants.TYPE_UNKNOWN){
			lastKnownFenceField.setY(y);
			return;
		}
	}

	@Override
	public int getPriority() {
		return 20;
	}

	@Override
	public boolean isAchieved() {
		return isReached;
	}

	@Override
	public String toString() {
		return "find switch for fence "+lastKnownFenceField;
	}

	@Override
	public void deserialise(IDeserialiser arg0) throws IOException,
			ClassNotFoundException {
		lastKnownFenceField = (Position)arg0.readObject();
		this.isReached = arg0.readBoolean();
	}

	@Override
	public void serialise(ISerialiser arg0) throws IOException {
		arg0.writeObject(lastKnownFenceField);
		arg0.writeBoolean(this.isReached);
	}

	@Override
	public Position getGoalPosition() {
		return lastKnownFenceField;
	}

}
