package de.tub.mapc.Coordination.goals;

import java.io.IOException;

import de.dailab.keks.io.IDeserialiser;
import de.dailab.keks.io.ISerialiser;
import de.tub.mapc.IMAPCConstants;
import de.tub.mapc.components.contestagents.MessageService;
import de.tub.mapc.components.contestagents.handle.IPathFindingHandle;
import de.tub.mapc.knowledge.Gridworld;
import de.tub.mapc.knowledge.Perception;
import de.tub.mapc.knowledge.Position;
import de.tub.mapc.knowledge.Switch;
import de.tub.mapc.pathfinding.PathResult;

public class GoalGoToSwitchingPosition extends AbstractGoal {
	
	// TODO es kann vorkommen, dass der selbst
	// an einem zaun warten muss, weil der am anfang
	// beim weg rechnen offen war und dann geschlossen wurde...
	// in dem fall unreachable senden?
	// ebenso wenn er selbst am switch steht und auf den zaun will... 
	// dann kommt er nicht weg...
	// wird er dann fuer einen anderen switch eingeteilt geht er da nie hin
	
	// kam beides oefter vor!
	
	
	
	private Position switchPosition;
	private Position goalPosition;
	private boolean isReached;
	private int remainingRounds;
	
	public GoalGoToSwitchingPosition(){
	}
	
	public GoalGoToSwitchingPosition(Position switchingPosition, Position switchPosition, int rounds) {
		goalPosition = switchingPosition;
		this.switchPosition = switchPosition;
		remainingRounds=rounds;
	}
	
	public Position getSwitchPosition() {
		return switchPosition;
	}

	public Position getGoalPosition() {
		return goalPosition;
	}

	@Override
	public String getAction(Perception perception, Gridworld world,	IPathFindingHandle pathfinder, MessageService coordinatorMessageService) {
		try {
			// pathfinder befragen
			PathResult pr = pathfinder.getPath(goalPosition, perception.getAgentsPosition(), world, false);
			if(pr.isReachedGoal()){
				
				// goal is reached, stay at hte switch until remainingRounds==0 and then until a new goal is received
				isReached = true;
				if(remainingRounds>0)remainingRounds--;
				return IMAPCConstants.ACTION_SKIP;
				
			}else if(pr.getErrortype() != null){
							
				if (pr.getErrortype() == IMAPCConstants.PATH_ERROR){
					// there was an real error in the pathfinder
					// this should not happen
					// but when, the agent should move on
					// -> random move
//					_logger.error("PATH: THERE WAS AN ERROR");
					return this.getRandomAction();
				}else if (pr.getErrortype() == IMAPCConstants.PATH_NOTFOUND){
					// the pathfinder was not able to find a path
					// should not happen, because this goal is only assigned if there is an path 
					return this.getRandomAction();
				
				}else if (pr.getErrortype() == IMAPCConstants.PATH_FENCEAHEAD){
					// the pathfinder indicates there is a fence in the next move,
					// maybe we already voted but the switcher hasn't reached the switching position yet or we didn't vote
					Switch s = world.findSwitchForFence(new Position(pr.getFencex(),pr.getFencey()));
					if(s == null){
						// there is no switch found yet
						coordinatorMessageService.sendSwitchVote(new Position(pr.getFencex(), pr.getFencey()),true, pr.getStepsToNextFence());
					}else{
						// vote for this switch to be opened
						coordinatorMessageService.sendSwitchVote(s, false, pr.getStepsToNextFence());
					}
					// If the fence is in the next move, the action is skip
					return pr.getThisStep();
				}else if (pr.getErrortype() == IMAPCConstants.PATH_BLOCKED){
					// The Pathfinder indicates the next move is blocked
					// da is ne kuh oder nen agent ins bild gerannt, // TODO randommove oder lieber skip?
					return this.getRandomAction();
				}
			}else{
				// no error, vote if there are fences ahead
				if(pr.getStepsToNextFence() != -1){
					//TODO ab welchem Abstand anmelden?
					Position s = world.findSwitchForFence(new Position(pr.getFencex(), pr.getFencey()));
					if(s == null){
						// there is no switch found yet
						coordinatorMessageService.sendSwitchVote(new Position(pr.getFencex(),pr.getFencey()),true, pr.getStepsToNextFence());
					}else{
						// vote for this switch to be opened
						coordinatorMessageService.sendSwitchVote(s, false, pr.getStepsToNextFence());
					}
				}
				//TODO weitere Zauune anmelden
				
				// und es koennte sein, dass keiner erreicht ohne durch Zaun zu gehen
				// koennen wir uns aber spaeter Gedanken drueben machen
				return pr.getThisStep();
			}
		}catch (Exception e){
			e.printStackTrace();
		}
		// hier nur nach exception, //TODO randommove oder lieber skip
		return this.getRandomAction();
	}

	@Override
	public void deserialise(IDeserialiser arg0) throws IOException,
			ClassNotFoundException {
		goalPosition = (Position)arg0.readObject();
		switchPosition = (Position)arg0.readObject();
		this.isReached = arg0.readBoolean();
	}

	@Override
	public void serialise(ISerialiser arg0) throws IOException {
		arg0.writeObject(goalPosition);
		arg0.writeObject(switchPosition);
		arg0.writeBoolean(this.isReached);
	}

	@Override
	public String toString() {
		return "go to switching position " + goalPosition;
	}

	@Override
	public int getPriority() {
		return 20;
	}

	@Override
	public boolean isAchieved() {
		return (isReached && remainingRounds==0);
	}

	@Override
	public boolean equals(Object o) {
		if(o == null){
			return false;
		}
		if(o instanceof GoalGoToSwitchingPosition){
			GoalGoToSwitchingPosition other = (GoalGoToSwitchingPosition) o;
			if (other.goalPosition.equals(goalPosition)){
				return true;
			}
		}
		return false;
	}

}
