package de.tub.mapc.components.contestagents;

import java.io.IOException;
import java.util.Enumeration;

import org.slf4j.Logger;

import de.dailab.keks.io.IAddress;
import de.dailab.keks.io.IMessage;
import de.jiac.micro.agent.AbstractReactiveBehaviour;
import de.jiac.micro.agent.handle.ICommunicationHandle;
import de.jiac.micro.agent.memory.ILongTermMemory;
import de.jiac.micro.core.IAgent;
import de.jiac.micro.core.feature.ILifecycleAware;
import de.jiac.micro.core.scope.AgentScope;
import de.tub.mapc.IMAPCConstants;
import de.tub.mapc.Coordination.goals.AbstractGoal;
import de.tub.mapc.components.contestagents.handle.IPathFindingHandle;
import de.tub.mapc.knowledge.Gridworld;
import de.tub.mapc.knowledge.Perception;
import de.tub.mapc.knowledge.SimulationEnd;
import de.tub.mapc.monitor.AgentMonitor;

public class SimulationResultElement extends AbstractReactiveBehaviour implements ILifecycleAware{

	private Logger _logger;
	private IAgent agent;
	private ILongTermMemory memory;
	private IPathFindingHandle pathfinder;
	private ICommunicationHandle _communicationHandle;
	private IAddress coordinatoraddress;
	
	@SuppressWarnings("unchecked")
	@Override
	protected Class[] filterDataTypes() {
		return new Class[] {SimulationEnd.class};
	}

	@SuppressWarnings("unchecked")
	@Override
	protected void runShort(Enumeration sensorReadings) {
		while (sensorReadings.hasMoreElements()) {
			Object read = sensorReadings.nextElement();
			if (read instanceof SimulationEnd){
				
				// ergebnis ausgeben
				_logger.info("RESULT OF THIS SIMULATION: "+((SimulationEnd)read).getResult());
				_logger.info("SCORE: "+((SimulationEnd)read).getScore());
				_logger.debug("a simend fact received" );
				// aufraeumen fuer naechste simulation 
				// also langzeitgedaechtnis, goals usw
					memory.forgetAll(AbstractGoal.class);
					memory.forgetAll(Gridworld.class);
					memory.forgetAll(Perception.class);
				// adressen fuer Kommunikation usw koennen erhalten bleiben 
				// Monitor und Coordinator muessen auch zurueckgesetzt werden, die 
				// nehmen nur eine neue world, wenn sie noch keine haben
				//-> send resetMessage -> setzen sich selbst zurueck
				sendResetMessage();
				// Wegfindungsalgorithmus neu initialisieren
				pathfinder.restart();
			}
		}
	}
	
	/**
	 * sends a message to the coordinator and to the monitor to inform them about the end of the simulation
	 */
	private void sendResetMessage() {
		IMessage message = _communicationHandle.createMessage();
		message.setContent(IMAPCConstants.MESSAGE_RESET);
		try {
			_communicationHandle.sendMessage(coordinatoraddress, message);
//			_communicationHandle.sendMessage(monitoragentaddress, message);
		} catch (IOException e) {
			_logger.error("Could not send the reset message to the coordinatoragent and/or the monitoragent", e);
		}
		
		// reset the monitor directly
		AgentMonitor.getInstance().reset();
	}

	public void start() {
        super.start();
        
		// Referenz auf den Agenten
		agent = AgentScope.getAgentReference();
		// Referenz auf den Logger
		_logger = agent.getLogger("SimulationResult");
		// reference to the longtermmemory
		memory = (ILongTermMemory) agent.getHandle(ILongTermMemory.class);
		// reference to the pathfinder
		pathfinder = (IPathFindingHandle) agent.getHandle(IPathFindingHandle.class);
		if (pathfinder == null)_logger.error("PATHFINDER IST NULL");
		// Referenz auf die Kommunikationsschnittstelle
		_communicationHandle = (ICommunicationHandle) agent.getHandle(ICommunicationHandle.class);
		//Monitor- und Coordinatoradresse
//		monitoragentaddress= _communicationHandle.getMulticastAddressForName("monitor");
		coordinatoraddress= _communicationHandle.getMulticastAddressForName("coordinator");
		
	}

}
