package de.tub.mapc.components.contestagents.handle;

import java.awt.Point;

import de.jiac.micro.core.IHandle;
import de.tub.mapc.knowledge.Gridworld;
import de.tub.mapc.knowledge.Position;
import de.tub.mapc.pathfinding.PathResult;

/**
 * This handle is used to access the server connector.
 * <code>IPathFindingHandle handle = (IPathFindingHandle) AgentScope.getAgentScope().getAgentReference().getHandle(IPathFindingHandle.class);</code>
 * 
 * @see de.jiac.micro.core.IHandle
 * 
 * @author Jonas Hartmann
 * @version $Revision:  $
 */
public interface IPathFindingHandle extends IHandle {
	/**
	 * Restart everything, including the world model
	 * @return <b>integer</b> <br/>
	 * <b>0</b> , everything is OK <br/>
	 * <b>-1</b> , indicates an error
	 */
	public int restart();

	/**
	 * computes a path using A* algorithm
	 * 
	 * @param goal the position of the field to reach
	 * @param start the agents position
	 * @param world 
	 * @param ignoreFences if false, fences are seen as obstacles
	 * @return
	 */
	PathResult getPath(Position goal, Position start, Gridworld world,
			boolean ignoreFences);

	/**
	 * computes a path using A* algorithm<br/>
	 * in this special version, cows are not seen as obstacles,
	 * so the path starts in the right direction to the corral
	 * 
	 * @param goal the position of the field to reach
	 * @param start the agents position
	 * @param world
	 * @return
	 */
	PathResult getPathForHerd(Position goal, Position start, Gridworld world);
	
	/**
	 * computes a path using A* algorithm<br/>
	 * in this special version, cows are not seen as obstacles,
	 * cause the blocker will go to the enemy corral and there are probably cows
	 * 
	 * @param goal the position of the field to reach
	 * @param start the agents position
	 * @param world
	 * @return
	 */
	PathResult getPathForBlocker(Position goal, Position start, Gridworld world);

	/**
	 * computes a path using A* algorithm<br/>
	 * in this special version, the whole herd is seen as an obstacle,
	 * so the driver does not walk through the herd
	 * @param goal the driving position to reach
	 * @param start the agents position
	 * @param world
	 * @param ul the upper left position of the herd
	 * @param lr the lower right position of the herd
	 * @return
	 */
	PathResult getPathForDriver(Position goal, Position start, Gridworld world,
			Point ul, Point lr); 
}
