package de.tub.mapc.parser;

import java.io.ByteArrayOutputStream;
import java.io.IOException;

import org.kxml2.io.KXmlSerializer;
import org.kxml2.kdom.Document;
import org.kxml2.kdom.Element;
import org.kxml2.kdom.Node;

/**
 * The message factory creates XML messages to be sent to the contest server.
 * 
 * @author Marcel Patzlaff
 * @author Erdene-Ochir Tuguldur
 * @version $Revision: $
 */
public class MAPCMessageFactory {
	private KXmlSerializer _serializer;

	public MAPCMessageFactory() {
		_serializer = new KXmlSerializer();
	}

	public Document createAction(String type, String param, String id) {
		Document document = new Document();
		document.setEncoding("UTF-8");
		//document.setEncoding("ISO-8859-1");
		Element root = document.createElement("", "message");
		root.setAttribute("", "type", "action");
		document.addChild(Node.ELEMENT, root);
		Element element = document.createElement("", "action");
		element.setAttribute("", "id", id);
		element.setAttribute("", "type", type);
		//element.setAttribute("", "param", param == null ? "" : param);
		root.addChild(Node.ELEMENT, element);
		return document;
	}

	public Document createAuthRequest(String username, String password) {
		Document document = new Document();
		document.setEncoding("UTF-8");
		//document.setEncoding("ISO-8859-1");
		Element root = document.createElement("", "message");
		root.setAttribute("", "type", "auth-request");
		document.addChild(Node.ELEMENT, root);
		Element element = document.createElement("", "authentication");
		element.setAttribute("", "username", username);
		element.setAttribute("", "password", password);
		root.addChild(Node.ELEMENT, element);
		return document;
	}

	public Document createPing(String payload) {
		Document document = new Document();
		document.setEncoding("UTF-8");
		//document.setEncoding("ISO-8859-1");
		Element root = document.createElement("", "message");
		root.setAttribute("", "type", "ping");
		document.addChild(Node.ELEMENT, root);
		Element element = document.createElement("", "payload");
		element.setAttribute("", "value", payload == null ? "" : payload);
		root.addChild(Node.ELEMENT, element);
		return document;
	}

	public synchronized byte[] serialiseXML(Document document) throws IOException {
		ByteArrayOutputStream buffer = new ByteArrayOutputStream();
		_serializer.setOutput(buffer, "UTF-8");
		//_serializer.setOutput(buffer, "ISO-8859-1");
		document.write(_serializer);
		return buffer.toByteArray();
	}
}
