package de.tub.mapc.pathfinding;

import static de.tub.mapc.monitor.MapComponent.COLORS;

import java.awt.Color;
import java.awt.Component;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.util.List;

import de.tub.mapc.worldmodel.GridField;

public class PathMap extends Component {
	// Booleans to control what to draw
	private boolean drawCost, drawPath;
	
	/**
	 * 
	 */
	private static final long serialVersionUID = -4965341891553823553L;
	private List<AStarNode> path;
	
	private GridField[][] world;
	private int imgsize = 6;
//	private State start, goal;
	private AStarNode start,goal;
	
	public PathMap() {
		super();
		drawCost = false;
		drawPath = true;
	}
	
//	public void setStart(State newStart){
//		this.start = newStart;
//		repaint();
//	}
//	
//	public void setGoal(State newGoal){
//		this.goal = newGoal;
//		repaint();
//	}
	public void cleanPath(){
		path.clear();
	}
	
	public void setStart(AStarNode newStart){
		this.start = newStart;
//		repaint();
	}
	
	public void setGoal(AStarNode newGoal){
		this.goal = newGoal;
//		repaint();
	}
	
//	public void update(GridField[][] world, HashMap<String,State> worldStates, State start, State goal) {
//		this.world = world;
//		this.allStates = worldStates;
//		setStart(start);
//		setGoal(goal);
//		repaint();
//	}
	public void update(GridField[][] world, List<AStarNode> path, AStarNode start, AStarNode goal) {
		this.world = world;
		this.path = path;
		setStart(start);
		setGoal(goal);
		repaint();
	}
	
	@SuppressWarnings("unchecked")
	public void update(GridField[][] world, List<AStarNode> path) {
		this.world = world;
		this.path = path;
		repaint();
	}
	
	@Override
	public void paint(Graphics g) {
		Graphics2D g2 = (Graphics2D)g;
		
		final int imgsize= Math.max(2,Math.min(getWidth() / world.length, getHeight() / world[0].length));
		for(int x = 0; x < world.length; x++) {
			for(int y = 0; y < world[0].length; y++) {
				if(world[x][y] != null) { 
					// First choose the color by the normal map
					int type = world[x][y].getType();
				    g2.setColor(COLORS[type]);
					g2.fillRect(imgsize*x, imgsize*y, imgsize-1, imgsize-1);//					
				}
			}
		} // end for
				
		if (drawPath){
			for (AStarNode next: path){
				if (next.equals(start)){
					g2.setColor(Color.BLACK);
					g2.fillRect(imgsize*next.getX(), imgsize*next.getY(), imgsize-1, imgsize-1);
				}else {
					g2.setColor(Color.YELLOW);
					g2.fillRect(imgsize*next.getX(), imgsize*next.getY(), imgsize-1, imgsize-1);
				}
				if (drawCost){
					char[] ca = String.valueOf(next.getCost()).toCharArray();
					g2.setColor(Color.BLACK);
					g2.drawChars(ca, 0, ca.length, imgsize*next.getX(), (imgsize*next.getY())+imgsize);
				}
			}
		}
		
		if (goal != null){
			g2.setColor(Color.DARK_GRAY);
			g2.fillRect(imgsize*goal.getX(), imgsize*goal.getY(), imgsize-1, imgsize-1);
		}
		if (start != null){
			g2.setColor(Color.BLACK);
			g2.fillRect(imgsize*start.getX(), imgsize*start.getY(), imgsize-1, imgsize-1);
		}
	}
}
